﻿Import("System.Windows.Application");
Import("System.Windows.Controls.UserControl");
Import("System.Windows.Controls.Canvas");
Import("System.Windows.Media.ScaleTransform");

var Model = {
  left: 0,
  top:  0,
  right:  500,
  bottom: 300,
  ballRadius: 26.0,
  maxSpeed:    3.0
};

function Ball(x, y, vx, vy) {
  // instance fields
  this.x = (x != null) ? x : ((Model.right - Model.left - 2 * Model.ballRadius) * Math.random());
  this.y = (y != null) ? y : ((Model.bottom - Model.top - 2 * Model.ballRadius) * Math.random());
  this.vx = (vx != null) ? vx : (2 * Model.maxSpeed * Math.random() - Model.maxSpeed);
  this.vy = (vy != null) ? vy : (2 * Model.maxSpeed * Math.random() - Model.maxSpeed);
  this.r = Model.ballRadius;
  this.d = 2 * this.r;
  this.d2 = this.d * this.d;
}
Ball.prototype = {
  // instance methods
  move: function() {
    this.x += this.vx;
    this.y += this.vy;
    // walls collisons

    // left
    if (this.x < Model.left && this.vx < 0) {
      this.vx = -this.vx;
    }
    // top
    if (this.y < Model.top && this.vy < 0) {
      this.vy = -this.vy;
    }
    // left
    if (this.x > Model.right - this.d && this.vx > 0) {
      this.vx = -this.vx;
    }
    // top
    if (this.y > Model.bottom - this.d && this.vy > 0) {
      this.vy = -this.vy;
    }
  },

  doCollide: function(b) {
    // calculate some vectors 
    var dx = this.x - b.x;
    var dy = this.y - b.y;
    var dvx = this.vx - b.vx;
    var dvy = this.vy - b.vy;
    var distance2 = dx * dx + dy * dy;

    if (Math.abs(dx) > this.d || Math.abs(dy) > this.d)
      return false;
    if (distance2 > this.d2)
      return false;

    // make absolutely elastic collision
    var mag = dvx * dx + dvy * dy;

    // test that balls move towards each other	
    if (mag > 0)
      return false;

    mag = mag / distance2;

    var deltavx = dx * mag;
    var deltavy = dy * mag;

    this.vx -= deltavx;
    this.vy -= deltavy;

    b.vx += deltavx;
    b.vy += deltavy;

    return true;
  }
};

function UIBall(x, y, vx, vy) {
  // instance fields
  this.$super = Ball;
  this.$super(x, y, vx, vy);
  this.content =  new UserControl();
  Application.Current.LoadComponent(this.content, "BallUserControl.xaml");
  this.rootCanvas = this.content.FindName("rootCanvas");
  this.setRadius(Model.ballRadius);
  this.$super = Ball.prototype.move;
  this.$super();
}
UIBall.prototype = {
  // instance methods
  move: function() {
    this.$super = Ball.prototype.move;
    this.$super();

    this.content.SetValue(Canvas.LeftProperty, this.x);
    this.content.SetValue(Canvas.TopProperty, this.y);
  },
  doCollide: Ball.prototype.doCollide,
  getBallControl: function() {
    return this.item;
  },
  setRadius: function(radius) {
    var trans = new ScaleTransform();
    trans.ScaleX = radius / 26.0;
    trans.ScaleY = radius / 26.0;
    this.rootCanvas.RenderTransform = trans;
  }
};
